<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\Base\Service\GetCurrentProductService As CurrentProduct;
use Flat101\EnhancedEcommerce\Model\DataLayer;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4;
use Magento\Store\Model\StoreManagerInterface;

class ProductView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'product-view-ga4';

    private const EXCLUDE_FIELDS = [
        'quantity',
        'position',
    ];

    /** @var Generator\Product */
    protected $productGeneratorGa4;

    /** @var Config */
    protected $configHelper;

    /** @var CurrentProduct */
    protected $currentProduct;

    /** @var StoreManagerInterface */
    protected $storeManager;

    public function __construct(
        DataLayerGa4\Generator\Product $productGeneratorGa4,
        Config $configHelper,
        CurrentProduct $currentProduct,
        StoreManagerInterface $storeManager
    ) {
        $this->productGeneratorGa4 = $productGeneratorGa4;
        $this->configHelper = $configHelper;
        $this->currentProduct = $currentProduct;
        $this->storeManager = $storeManager;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    public function getLayer(): ?DataLayer\Dto\Ecommerce
    {
        $currentProduct = $this->currentProduct->getProduct();
        $product = $this->productGeneratorGa4->generate($currentProduct);

        if (is_null($product)) {
            return null;
        }

        $this->unset($product, self::EXCLUDE_FIELDS);

        $detailEcommerceDto = new DataLayerGa4\Dto\EcommerceDetails();
        $detailEcommerceDto->currency = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $detailEcommerceDto->items[] = $product;

        $ecommerce = new DataLayer\Dto\Ecommerce();
        $ecommerce->event = 'view_item';
        $ecommerce->ecommerce = $detailEcommerceDto;

        $this->unset($ecommerce, ['eventCategory','eventAction','eventLabel']);

        return $ecommerce;
    }
}
