<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Observer;

use Flat101\EnhancedEcommerce\Factory\ImpressionHandlerFactory;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Framework\App\Request\Http;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Psr\Log\LoggerInterface;
use Magento\Catalog\Block\Product\ProductList\Toolbar;

class CatalogBlockProductListCollectionObserver implements ObserverInterface
{
    /** @var Http|RequestInterface */
    private $request;

    /** @var ImpressionHandlerFactory */
    private $impressionHandlerFactory;

    /** @var Toolbar */
    protected $toolbar;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        RequestInterface $request,
        ImpressionHandlerFactory $impressionHandlerFactory,
        Toolbar $toolbar,
        LoggerInterface $logger
    ) {
        $this->request = $request;
        $this->impressionHandlerFactory = $impressionHandlerFactory;
        $this->toolbar = $toolbar;
        $this->logger = $logger;
    }

    public function execute(Observer $observer): void
    {
        try {
            /** @var Collection $collection */
            $collection = $observer->getEvent()->getData('collection');

            // to get current sort by
            $productOrder = $this->toolbar->getCurrentOrder();
            // to get current sort direction
            $productOrderDirection = $this->toolbar->getCurrentDirection();

            $pageSize = $this->toolbar->getLimit();
            $pageNumber = $this->toolbar->getCurrentPage();

            $collection->setOrder($productOrder, $productOrderDirection);
            $collection->setCurPage($pageNumber)->setPageSize($pageSize);

            $handlerImpression = $this->impressionHandlerFactory->create($this->request->getRouteName());

            if (null !== $handlerImpression) {
                $handlerImpression->handle($collection);
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }
    }
}
